@@ -29,6 +29,7 @@
 
 /* ### this should go away, but it causes too much breakage right now */
 #include <stdlib.h>
+#include <limits.h> /* for ULONG_MAX */
 
 #include <apr.h>         /* for apr_size_t, apr_int64_t, ... */
 #include <apr_errno.h>   /* for apr_status_t */
@@ -71,16 +72,38 @@ extern "C" {
  * Unaligned access on other machines (e.g. IA64) will trigger memory
  * access faults or simply misbehave.
  *
+ * Note: Some platforms may only support unaligned access for integers
+ * (PowerPC).  As a result this macro should only be used to determine
+ * if unaligned access is supported for integers.
+ *
  * @since New in 1.7.
  */
 #ifndef SVN_UNALIGNED_ACCESS_IS_OK
-# if defined(_M_IX86) || defined(_M_X64) || defined(i386) || defined(__x86_64)
+# if defined(_M_IX86) || defined(i386) \
+     || defined(_M_X64) || defined(__x86_64) \
+     || defined(__powerpc__) || defined(__ppc__)
 #  define SVN_UNALIGNED_ACCESS_IS_OK 1
 # else
 #  define SVN_UNALIGNED_ACCESS_IS_OK 0
 # endif
 #endif
 
+
+
+/** YABT:  Yet Another Boolean Type */
+typedef int svn_boolean_t;
+
+#ifndef TRUE
+/** uhh... true */
+#define TRUE 1
+#endif /* TRUE */
+
+#ifndef FALSE
+/** uhh... false */
+#define FALSE 0
+#endif /* FALSE */
+
+
 
 /** Subversion error object.
  *
@@ -131,9 +154,13 @@ typedef struct svn_error_t
 
 } svn_error_t;
 
+
+
 /* See svn_version.h.
    Defined here to avoid including svn_version.h from all public headers. */
 typedef struct svn_version_t svn_version_t;
+
+
 
 /** @defgroup APR_ARRAY_compat_macros APR Array Compatibility Helper Macros
  * These macros are provided by APR itself from version 1.3.
@@ -152,6 +179,8 @@ typedef struct svn_version_t svn_version_t;
 #endif
 
 /** @} */
+
+
 
 /** @defgroup apr_hash_utilities APR Hash Table Helpers
  * These functions enable the caller to dereference an APR hash table index
@@ -173,6 +202,10 @@ svn__apr_hash_index_klen(const apr_hash_index_t *hi);
 void *
 svn__apr_hash_index_val(const apr_hash_index_t *hi);
 
+/** @} */
+
+
+
 /** On Windows, APR_STATUS_IS_ENOTDIR includes several kinds of
  * invalid-pathname error but not ERROR_INVALID_NAME, so we include it.
  * We also include ERROR_DIRECTORY as that was not included in apr versions
@@ -197,6 +230,8 @@ svn__apr_hash_index_val(const apr_hash_index_t *hi);
 #endif
 
 /** @} */
+
+
 
 /** The various types of nodes in the Subversion filesystem. */
 typedef enum svn_node_kind_t
@@ -211,7 +246,14 @@ typedef enum svn_node_kind_t
   svn_node_dir,
 
   /** something's here, but we don't know what */
-  svn_node_unknown
+  svn_node_unknown,
+
+  /**
+   * symbolic link
+   * @note This value is not currently used by the public API.
+   * @since New in 1.8.
+   */
+  svn_node_symlink
 } svn_node_kind_t;
 
 /** Return a constant string expressing @a kind as an English word, e.g.,
@@ -233,6 +275,7 @@ svn_node_kind_to_word(svn_node_kind_t kind);
 svn_node_kind_t
 svn_node_kind_from_word(const char *word);
 
+
 /** Generic three-state property to represent an unknown value for values
  * that are just like booleans.  The values have been set deliberately to
  * make tristates disjoint from #svn_boolean_t.
@@ -243,8 +286,11 @@ svn_node_kind_from_word(const char *word);
  * @since New in 1.7. */
 typedef enum svn_tristate_t
 {
+  /** state known to be false (the constant does not evaulate to false) */
   svn_tristate_false = 2,
+  /** state known to be true */
   svn_tristate_true,
+  /** state could be true or false */
   svn_tristate_unknown
 } svn_tristate_t;
 
@@ -266,6 +312,7 @@ svn_tristate_t
 svn_tristate__from_word(const char * word);
 
 
+
 /** About Special Files in Subversion
  *
  * Subversion denotes files that cannot be portably created or
@@ -303,6 +350,8 @@ svn_tristate__from_word(const char * word);
  *     routines from 1.
  */
 
+
+
 /** A revision number. */
 typedef long int svn_revnum_t;
 
@@ -349,6 +398,7 @@ svn_revnum_parse(svn_revnum_t *rev,
 #define SVN_REVNUM_T_FMT "ld"
 
 
+
 /** The size of a file in the Subversion FS. */
 typedef apr_int64_t svn_filesize_t;
 
@@ -366,20 +416,7 @@ typedef apr_int64_t svn_filesize_t;
 #endif
 
 
-/** YABT:  Yet Another Boolean Type */
-typedef int svn_boolean_t;
-
-#ifndef TRUE
-/** uhh... true */
-#define TRUE 1
-#endif /* TRUE */
-
-#ifndef FALSE
-/** uhh... false */
-#define FALSE 0
-#endif /* FALSE */
-
-
+
 /** An enum to indicate whether recursion is needed. */
 enum svn_recurse_kind
 {
@@ -437,7 +474,6 @@ typedef enum svn_depth_t
 
 } svn_depth_t;
 
-
 /** Return a constant string expressing @a depth as an English word,
  * e.g., "infinity", "immediates", etc.  The string is not localized,
  * as it may be used for client<->server communications.
@@ -447,7 +483,6 @@ typedef enum svn_depth_t
 const char *
 svn_depth_to_word(svn_depth_t depth);
 
-
 /** Return the appropriate depth for @a depth_str.  @a word is as
  * returned from svn_depth_to_word().  If @a depth_str does not
  * represent a recognized depth, return #svn_depth_unknown.
@@ -457,8 +492,7 @@ svn_depth_to_word(svn_depth_t depth);
 svn_depth_t
 svn_depth_from_word(const char *word);
 
-
-/* Return #svn_depth_infinity if boolean @a recurse is TRUE, else
+/** Return #svn_depth_infinity if boolean @a recurse is TRUE, else
  * return #svn_depth_files.
  *
  * @note New code should never need to use this, it is called only
@@ -469,8 +503,7 @@ svn_depth_from_word(const char *word);
 #define SVN_DEPTH_INFINITY_OR_FILES(recurse) \
   ((recurse) ? svn_depth_infinity : svn_depth_files)
 
-
-/* Return #svn_depth_infinity if boolean @a recurse is TRUE, else
+/** Return #svn_depth_infinity if boolean @a recurse is TRUE, else
  * return #svn_depth_immediates.
  *
  * @note New code should never need to use this, it is called only
@@ -481,8 +514,7 @@ svn_depth_from_word(const char *word);
 #define SVN_DEPTH_INFINITY_OR_IMMEDIATES(recurse) \
   ((recurse) ? svn_depth_infinity : svn_depth_immediates)
 
-
-/* Return #svn_depth_infinity if boolean @a recurse is TRUE, else
+/** Return #svn_depth_infinity if boolean @a recurse is TRUE, else
  * return #svn_depth_empty.
  *
  * @note New code should never need to use this, it is called only
@@ -493,8 +525,7 @@ svn_depth_from_word(const char *word);
 #define SVN_DEPTH_INFINITY_OR_EMPTY(recurse) \
   ((recurse) ? svn_depth_infinity : svn_depth_empty)
 
-
-/* Return a recursion boolean based on @a depth.
+/** Return a recursion boolean based on @a depth.
  *
  * Although much code has been converted to use depth, some code still
  * takes a recurse boolean.  In most cases, it makes sense to treat
@@ -502,10 +533,10 @@ svn_depth_from_word(const char *word);
  * non-recursive (which in turn usually translates to #svn_depth_files).
  */
 #define SVN_DEPTH_IS_RECURSIVE(depth)                              \
-  (((depth) == svn_depth_infinity || (depth) == svn_depth_unknown) \
-   ? TRUE : FALSE)
+  ((depth) == svn_depth_infinity || (depth) == svn_depth_unknown)
 
 
+
 /**
  * It is sometimes convenient to indicate which parts of an #svn_dirent_t
  * object you are actually interested in, so that calculating and sending
@@ -539,7 +570,13 @@ svn_depth_from_word(const char *word);
 
 /** @} */
 
-/** A general subversion directory entry. */
+/** A general subversion directory entry.
+ *
+ * @note To allow for extending the #svn_dirent_t structure in future
+ * releases, always use svn_dirent_create() to allocate the stucture.
+ *
+ * @since New in 1.6.
+ */
 typedef struct svn_dirent_t
 {
   /** node kind */
@@ -563,7 +600,6 @@ typedef struct svn_dirent_t
   /* IMPORTANT: If you extend this struct, check svn_dirent_dup(). */
 } svn_dirent_t;
 
-
 /** Return a deep copy of @a dirent, allocated in @a pool.
  *
  * @since New in 1.4.
@@ -572,8 +608,16 @@ svn_dirent_t *
 svn_dirent_dup(const svn_dirent_t *dirent,
                apr_pool_t *pool);
 
-
+/**
+ * Create a new svn_dirent_t instance with all values initialized to their
+ * not-available values.
+ *
+ * @since New in 1.8.
+ */
+svn_dirent_t *
+svn_dirent_create(apr_pool_t *result_pool);
 
+
 /** Keyword substitution.
  *
  * All the keywords Subversion recognizes.
@@ -650,6 +694,7 @@ svn_dirent_dup(const svn_dirent_t *dirent,
 
 /** @} */
 
+
 
 /** All information about a commit.
  *
@@ -678,7 +723,6 @@ typedef struct svn_commit_info_t
 
 } svn_commit_info_t;
 
-
 /**
  * Allocate an object of type #svn_commit_info_t in @a pool and
  * return it.
@@ -696,7 +740,6 @@ typedef struct svn_commit_info_t
 svn_commit_info_t *
 svn_create_commit_info(apr_pool_t *pool);
 
-
 /**
  * Return a deep copy @a src_commit_info allocated in @a pool.
  *
@@ -706,6 +749,7 @@ svn_commit_info_t *
 svn_commit_info_dup(const svn_commit_info_t *src_commit_info,
                     apr_pool_t *pool);
 
+
 
 /**
  * A structure to represent a path that changed for a log entry.
@@ -767,7 +811,7 @@ svn_log_changed_path2_dup(const svn_log_changed_path2_t *changed_path,
 
 /**
  * A structure to represent a path that changed for a log entry.  Same as
- * #svn_log_changed_path2_t, but without the node kind.
+ * the first three fields of #svn_log_changed_path2_t.
  *
  * @deprecated Provided for backward compatibility with the 1.5 API.
  */
@@ -784,7 +828,6 @@ typedef struct svn_log_changed_path_t
 
 } svn_log_changed_path_t;
 
-
 /**
  * Return a deep copy of @a changed_path, allocated in @a pool.
  *
@@ -820,7 +863,8 @@ typedef struct svn_log_entry_t
   svn_revnum_t revision;
 
   /** The hash of requested revision properties, which may be NULL if it
-   * would contain no revprops. */
+   * would contain no revprops.  Maps (const char *) property name to
+   * (svn_string_t *) property value. */
   apr_hash_t *revprops;
 
   /**
@@ -862,6 +906,10 @@ typedef struct svn_log_entry_t
    * Whether @a revision should be interpreted as non-inheritable in the
    * same sense of #svn_merge_range_t.
    *
+   * Only set when this #svn_log_entry_t instance is returned by the
+   * libsvn_client mergeinfo apis. Currently always FALSE when the
+   * #svn_log_entry_t instance is reported by the ra layer.
+   *
    * @since New in 1.7.
    */
   svn_boolean_t non_inheritable;
@@ -933,7 +981,6 @@ svn_log_entry_dup(const svn_log_entry_t *log_entry, apr_pool_t *pool);
  *
  * @since New in 1.5.
  */
-
 typedef svn_error_t *(*svn_log_entry_receiver_t)(
   void *baton,
   svn_log_entry_t *log_entry,
@@ -954,6 +1001,7 @@ typedef svn_error_t *(*svn_log_message_receiver_t)(
   const char *message,
   apr_pool_t *pool);
 
+
 
 /** Callback function type for commits.
  *
@@ -979,6 +1027,7 @@ typedef svn_error_t *(*svn_commit_callback_t)(
   const char *author,
   void *baton);
 
+
 
 /** A buffer size that may be used when processing a stream of data.
  *
@@ -1031,7 +1080,6 @@ svn_error_t *
 svn_mime_type_validate(const char *mime_type,
                        apr_pool_t *pool);
 
-
 /** Return FALSE iff @a mime_type is a textual type.
  *
  * All mime types that start with "text/" are textual, plus some special
@@ -1102,6 +1150,8 @@ svn_lock_create(apr_pool_t *pool);
 svn_lock_t *
 svn_lock_dup(const svn_lock_t *lock, apr_pool_t *pool);
 
+
+
 /**
  * Return a formatted Universal Unique IDentifier (UUID) string.
  *
@@ -1110,6 +1160,8 @@ svn_lock_dup(const svn_lock_t *lock, apr_pool_t *pool);
 const char *
 svn_uuid_generate(apr_pool_t *pool);
 
+
+
 /**
  * Mergeinfo representing a merge of a range of revisions.
  *
@@ -1156,7 +1208,7 @@ svn_merge_range_contains_rev(const svn_merge_range_t *range, svn_revnum_t rev);
  *  @{ */
 
 /**
- * A representation of a segment of a object's version history with an
+ * A representation of a segment of an object's version history with an
  * emphasis on the object's location in the repository as of various
  * revisions.
  *
@@ -1165,7 +1217,7 @@ svn_merge_range_contains_rev(const svn_merge_range_t *range, svn_revnum_t rev);
 typedef struct svn_location_segment_t
 {
   /** The beginning (oldest) and ending (youngest) revisions for this
-      segment. */
+      segment, both inclusive. */
   svn_revnum_t range_start;
   svn_revnum_t range_end;
 
@@ -1175,7 +1227,6 @@ typedef struct svn_location_segment_t
 
 } svn_location_segment_t;
 
-
 /**
  * A callback invoked by generators of #svn_location_segment_t
  * objects, used to report information about a versioned object's
@@ -1187,7 +1238,6 @@ typedef svn_error_t *(*svn_location_segment_receiver_t)(
   void *baton,
   apr_pool_t *pool);
 
-
 /**
  * Return a deep copy of @a segment, allocated in @a pool.
  *
@@ -1199,19 +1249,22 @@ svn_location_segment_dup(const svn_location_segment_t *segment,
 
 /** @} */
 
+
+
 /** A line number, such as in a file or a stream.
  *
  * @since New in 1.7.
  */
 typedef unsigned long svn_linenum_t;
 
-/* The maximum value of an svn_linenum_t.
+/** The maximum value of an svn_linenum_t.
  *
  * @since New in 1.7.
  */
 #define SVN_LINENUM_MAX_VALUE ULONG_MAX
 
 
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
