@@ -89,7 +89,7 @@ put_copy(svn_fs_t *fs,
   svn_fs_base__str_to_dbt(&key, copy_id);
   svn_fs_base__skel_to_dbt(&value, copy_skel, pool);
   svn_fs_base__trail_debug(trail, "copies", "put");
-  return BDB_WRAP(fs, _("storing copy record"),
+  return BDB_WRAP(fs, N_("storing copy record"),
                   bfd->copies->put(bfd->copies, trail->db_txn,
                                    &key, &value, 0));
 }
@@ -112,7 +112,7 @@ svn_fs_bdb__reserve_copy_id(const char **id_p,
   /* Get the current value associated with the `next-key' key in the
      copies table.  */
   svn_fs_base__trail_debug(trail, "copies", "get");
-  SVN_ERR(BDB_WRAP(fs, _("allocating new copy ID (getting 'next-key')"),
+  SVN_ERR(BDB_WRAP(fs, N_("allocating new copy ID (getting 'next-key')"),
                    bfd->copies->get(bfd->copies, trail->db_txn, &query,
                                     svn_fs_base__result_dbt(&result),
                                     0)));
@@ -130,7 +130,7 @@ svn_fs_bdb__reserve_copy_id(const char **id_p,
                             svn_fs_base__str_to_dbt(&result, next_key),
                             0);
 
-  return BDB_WRAP(fs, _("bumping next copy key"), db_err);
+  return BDB_WRAP(fs, N_("bumping next copy key"), db_err);
 }
 
 
@@ -168,7 +168,7 @@ svn_fs_bdb__delete_copy(svn_fs_t *fs,
   db_err = bfd->copies->del(bfd->copies, trail->db_txn, &key, 0);
   if (db_err == DB_NOTFOUND)
     return svn_fs_base__err_no_such_copy(fs, copy_id);
-  return BDB_WRAP(fs, _("deleting entry from 'copies' table"), db_err);
+  return BDB_WRAP(fs, N_("deleting entry from 'copies' table"), db_err);
 }
 
 
@@ -196,7 +196,7 @@ svn_fs_bdb__get_copy(copy_t **copy_p,
 
   if (db_err == DB_NOTFOUND)
     return svn_fs_base__err_no_such_copy(fs, copy_id);
-  SVN_ERR(BDB_WRAP(fs, _("reading copy"), db_err));
+  SVN_ERR(BDB_WRAP(fs, N_("reading copy"), db_err));
 
   /* Unparse COPY skel */
   skel = svn_skel__parse(value.data, value.size, pool);
