@@ -28,6 +28,7 @@
 
 #define SVN_DEPRECATED
 
+#include "svn_hash.h"
 #include "svn_pools.h"
 #include "svn_types.h"
 #include "svn_mergeinfo.h"
@@ -78,7 +79,7 @@ verify_mergeinfo_parse(const char *input,
     {
       const void *path;
       void *val;
-      apr_array_header_t *ranges;
+      svn_rangelist_t *ranges;
       svn_merge_range_t *range;
       int j;
 
@@ -236,7 +237,7 @@ test_mergeinfo_dup(apr_pool_t *pool)
 {
   apr_hash_t *orig_mergeinfo, *copied_mergeinfo;
   apr_pool_t *subpool;
-  apr_array_header_t *rangelist;
+  svn_rangelist_t *rangelist;
 
   /* Assure that copies which should be empty turn out that way. */
   subpool = svn_pool_create(pool);
@@ -399,7 +400,7 @@ range_to_string(svn_merge_range_t *range,
    verified (e.g. "svn_rangelist_intersect"), while TYPE is a word
    describing what the ranges being examined represent. */
 static svn_error_t *
-verify_ranges_match(const apr_array_header_t *actual_rangelist,
+verify_ranges_match(const svn_rangelist_t *actual_rangelist,
                     svn_merge_range_t *expected_ranges, int nbr_expected,
                     const char *func_verified, const char *type,
                     apr_pool_t *pool)
@@ -436,7 +437,7 @@ verify_mergeinfo_deltas(apr_hash_t *deltas, svn_merge_range_t *expected_deltas,
                         const char *func_verified, const char *type,
                         apr_pool_t *pool)
 {
-  apr_array_header_t *rangelist;
+  svn_rangelist_t *rangelist;
 
   if (apr_hash_count(deltas) != 1)
     /* Deltas on "/trunk" expected. */
@@ -479,12 +480,11 @@ test_diff_mergeinfo(apr_pool_t *pool)
 static svn_error_t *
 test_rangelist_reverse(apr_pool_t *pool)
 {
-  apr_array_header_t *rangelist;
+  svn_rangelist_t *rangelist;
   svn_merge_range_t expected_rangelist[3] =
     { {10, 9, TRUE}, {7, 4, TRUE}, {3, 2, TRUE} };
 
-  SVN_ERR(svn_mergeinfo_parse(&info1, "/trunk: 3,5-7,10", pool));
-  rangelist = apr_hash_get(info1, "/trunk", APR_HASH_KEY_STRING);
+  SVN_ERR(svn_rangelist__parse(&rangelist, "3,5-7,10", pool));
 
   SVN_ERR(svn_rangelist_reverse(rangelist, pool));
 
@@ -495,7 +495,7 @@ test_rangelist_reverse(apr_pool_t *pool)
 static svn_error_t *
 test_rangelist_intersect(apr_pool_t *pool)
 {
-  apr_array_header_t *rangelist1, *rangelist2, *intersection;
+  svn_rangelist_t *rangelist1, *rangelist2, *intersection;
 
   /* Expected intersection when considering inheritance. */
   svn_merge_range_t intersection_consider_inheritance[] =
@@ -506,11 +506,9 @@ test_rangelist_intersect(apr_pool_t *pool)
     { {0, 1, TRUE}, {2, 4, TRUE}, {11, 12, TRUE}, {30, 32, FALSE},
       {39, 42, TRUE} };
 
-  SVN_ERR(svn_mergeinfo_parse(&info1, "/trunk: 1-6,12-16,30-32*,40-42", pool));
-  SVN_ERR(svn_mergeinfo_parse(&info2, "/trunk: 1,3-4*,7,9,11-12,31-34*,38-44",
-                              pool));
-  rangelist1 = apr_hash_get(info1, "/trunk", APR_HASH_KEY_STRING);
-  rangelist2 = apr_hash_get(info2, "/trunk", APR_HASH_KEY_STRING);
+  SVN_ERR(svn_rangelist__parse(&rangelist1, "1-6,12-16,30-32*,40-42", pool));
+  SVN_ERR(svn_rangelist__parse(&rangelist2, "1,3-4*,7,9,11-12,31-34*,38-44",
+                               pool));
 
   /* Check the intersection while considering inheritance twice, reversing
      the order of the rangelist arguments on the second call to
@@ -565,7 +563,7 @@ test_mergeinfo_intersect(apr_pool_t *pool)
 {
   svn_merge_range_t expected_intersection[3] =
     { {0, 1, TRUE}, {2, 4, TRUE}, {11, 12, TRUE} };
-  apr_array_header_t *rangelist;
+  svn_rangelist_t *rangelist;
   apr_hash_t *intersection;
 
   SVN_ERR(svn_mergeinfo_parse(&info1, "/trunk: 1-6,12-16\n/foo: 31", pool));
@@ -605,6 +603,7 @@ test_merge_mergeinfo(apr_pool_t *pool)
   struct mergeinfo_merge_path_range
     {
       const char *path;
+      int expected_n;
       svn_merge_range_t expected_rngs[MAX_NBR_MERGEINFO_RANGES];
     };
 
@@ -627,75 +626,75 @@ test_merge_mergeinfo(apr_pool_t *pool)
       /* One path, intersecting inheritable ranges */
       { "/trunk: 5-10",
         "/trunk: 6", 1,
-        { {"/trunk", { {4, 10, TRUE} } } } },
+        { {"/trunk", 1, { {4, 10, TRUE} } } } },
 
       /* One path, intersecting non-inheritable ranges */
       { "/trunk: 5-10*",
         "/trunk: 6*", 1,
-        { {"/trunk", { {4, 10, FALSE} } } } },
+        { {"/trunk", 1, { {4, 10, FALSE} } } } },
 
       /* One path, intersecting ranges with different inheritability */
       { "/trunk: 5-10",
         "/trunk: 6*", 1,
-        { {"/trunk", { {4, 10, TRUE} } } } },
+        { {"/trunk", 1, { {4, 10, TRUE} } } } },
 
       /* One path, intersecting ranges with different inheritability */
       { "/trunk: 5-10*",
         "/trunk: 6", 1,
-        { {"/trunk", { {4, 5, FALSE}, {5, 6, TRUE}, {6, 10, FALSE} } } } },
+        { {"/trunk", 3, { {4, 5, FALSE}, {5, 6, TRUE}, {6, 10, FALSE} } } } },
 
       /* Adjacent ranges all inheritable ranges */
       { "/trunk: 1,3,5-11,13",
         "/trunk: 2,4,12,14-22", 1,
-         { {"/trunk", { {0, 22, TRUE} } } } },
+        { {"/trunk", 1, { {0, 22, TRUE} } } } },
 
       /* Adjacent ranges all non-inheritable ranges */
       { "/trunk: 1*,3*,5-11*,13*",
         "/trunk: 2*,4*,12*,14-22*", 1,
-         { {"/trunk", { {0, 22, FALSE} } } } },
+        { {"/trunk", 1, { {0, 22, FALSE} } } } },
 
       /* Adjacent ranges differing inheritability */
       { "/trunk: 1*,3*,5-11*,13*",
         "/trunk: 2,4,12,14-22", 1,
-         { {"/trunk", { { 0,  1, FALSE}, { 1,  2, TRUE},
-                        { 2,  3, FALSE}, { 3,  4, TRUE},
-                        { 4, 11, FALSE}, {11, 12, TRUE},
-                        {12, 13, FALSE}, {13, 22, TRUE} } } } },
+        { {"/trunk", 8, { { 0,  1, FALSE}, { 1,  2, TRUE},
+                          { 2,  3, FALSE}, { 3,  4, TRUE},
+                          { 4, 11, FALSE}, {11, 12, TRUE},
+                          {12, 13, FALSE}, {13, 22, TRUE} } } } },
 
       /* Adjacent ranges differing inheritability */
       { "/trunk: 1,3,5-11,13",
         "/trunk: 2*,4*,12*,14-22*", 1,
-         { {"/trunk", { { 0,  1, TRUE}, { 1,  2, FALSE},
-                        { 2,  3, TRUE}, { 3,  4, FALSE},
-                        { 4, 11, TRUE}, {11, 12, FALSE},
-                        {12, 13, TRUE}, {13, 22, FALSE} } } } },
+        { {"/trunk", 8, { { 0,  1, TRUE}, { 1,  2, FALSE},
+                          { 2,  3, TRUE}, { 3,  4, FALSE},
+                          { 4, 11, TRUE}, {11, 12, FALSE},
+                          {12, 13, TRUE}, {13, 22, FALSE} } } } },
 
       /* Two paths all inheritable ranges */
       { "/trunk::1: 3,5,7-9,10,11,13,14\n/fred:8-10",
         "/trunk::1: 1-4,6\n/fred:9-12", 2,
-        { {"/trunk::1", { {0, 11, TRUE}, {12, 14, TRUE} } },
-          {"/fred",  { {7, 12, TRUE} } } } },
+        { {"/trunk::1", 2, { {0, 11, TRUE}, {12, 14, TRUE} } },
+          {"/fred",     1, { {7, 12, TRUE} } } } },
 
       /* Two paths all non-inheritable ranges */
       { "/trunk: 3*,5*,7-9*,10*,11*,13*,14*\n/fred:8-10*",
         "/trunk: 1-4*,6*\n/fred:9-12*", 2,
-        { {"/trunk", { {0, 11, FALSE}, {12, 14, FALSE} } },
-          {"/fred",  { {7, 12, FALSE} } } } },
+        { {"/trunk", 2, { {0, 11, FALSE}, {12, 14, FALSE} } },
+          {"/fred",  1, { {7, 12, FALSE} } } } },
 
       /* Two paths mixed inheritability */
       { "/trunk: 3,5*,7-9,10,11*,13,14\n/fred:8-10",
         "/trunk: 1-4,6\n/fred:9-12*", 2,
-        { {"/trunk", { { 0,  4, TRUE }, { 4,  5, FALSE}, {5, 10, TRUE},
-                       {10, 11, FALSE}, {12, 14, TRUE } } },
-          {"/fred",  { { 7, 10, TRUE }, {10, 12, FALSE} } } } },
+        { {"/trunk", 5, { { 0,  4, TRUE }, { 4,  5, FALSE}, {5, 10, TRUE},
+                          {10, 11, FALSE}, {12, 14, TRUE } } },
+          {"/fred",  2, { { 7, 10, TRUE }, {10, 12, FALSE} } } } },
 
       /* A slew of different paths but no ranges to be merged */
       { "/trunk: 3,5-9*\n/betty: 2-4",
         "/fred: 1-18\n/:barney: 1,3-43", 4,
-        { {"/trunk",   { {2,  3, TRUE}, {4,  9, FALSE} } },
-          {"/betty",   { {1,  4, TRUE} } },
-          {"/:barney", { {0,  1, TRUE}, {2, 43, TRUE} } },
-          {"/fred",    { {0, 18, TRUE} } } } }
+        { {"/trunk",   2, { {2,  3, TRUE}, {4,  9, FALSE} } },
+          {"/betty",   1, { {1,  4, TRUE} } },
+          {"/:barney", 2, { {0,  1, TRUE}, {2, 43, TRUE} } },
+          {"/fred",    1, { {0, 18, TRUE} } } } }
     };
 
   for (i = 0; i < NBR_MERGEINFO_MERGES; i++)
@@ -714,7 +713,7 @@ test_merge_mergeinfo(apr_pool_t *pool)
       SVN_ERR(svn_mergeinfo_to_string(&info2_starting, info2, pool));
 
       SVN_ERR(svn_mergeinfo_merge(info1, info2, pool));
-      if (mergeinfo[i].expected_paths != apr_hash_count(info1))
+      if (mergeinfo[i].expected_paths != (int)apr_hash_count(info1))
         return fail(pool, "Wrong number of paths in merged mergeinfo");
 
       /* Check that info2 remained unchanged. */
@@ -730,36 +729,18 @@ test_merge_mergeinfo(apr_pool_t *pool)
 
       for (j = 0; j < mergeinfo[i].expected_paths; j++)
         {
-          int k;
-          apr_array_header_t *rangelist =
+          svn_rangelist_t *rangelist =
             apr_hash_get(info1, mergeinfo[i].path_rngs[j].path,
                          APR_HASH_KEY_STRING);
           if (!rangelist)
             return fail(pool, "Missing path '%s' in merged mergeinfo",
-                        mergeinfo[i].path_rngs->path);
-          for (k = 0; k < rangelist->nelts; k++)
-            {
-              svn_merge_range_t *ranges =
-                APR_ARRAY_IDX(rangelist, k, svn_merge_range_t *);
-              if (ranges->start
-                    != mergeinfo[i].path_rngs[j].expected_rngs[k].start
-                  || ranges->end
-                    != mergeinfo[i].path_rngs[j].expected_rngs[k].end
-                  || ranges->inheritable
-                    != mergeinfo[i].path_rngs[j].expected_rngs[k].inheritable)
-                return fail(
-                  pool,
-                  "Range'%i-%i%s' not found in merged mergeinfo",
-                  mergeinfo[i].path_rngs->expected_rngs[k].start,
-                  mergeinfo[i].path_rngs->expected_rngs[k].end,
-                  mergeinfo[i].path_rngs->expected_rngs[k].inheritable
-                  ? "" : "*");
-            }
-          /* Were more ranges expected? */
-          if (k < MAX_NBR_MERGEINFO_RANGES
-              && mergeinfo[i].path_rngs[j].expected_rngs[k].start != 0)
-            return fail(pool,
-                        "Not all expected ranges found in merged mergeinfo");
+                        mergeinfo[i].path_rngs[j].path);
+          SVN_ERR(verify_ranges_match(
+                    rangelist,
+                    mergeinfo[i].path_rngs[j].expected_rngs,
+                    mergeinfo[i].path_rngs[j].expected_n,
+                    apr_psprintf(pool, "svn_rangelist_merge case %i:%i", i, j),
+                    "merge", pool));
         }
     }
 
@@ -771,7 +752,7 @@ test_remove_rangelist(apr_pool_t *pool)
 {
   int i, j;
   svn_error_t *err, *child_err;
-  apr_array_header_t *output, *eraser, *whiteboard;
+  svn_rangelist_t *output, *eraser, *whiteboard;
 
   /* Struct for svn_rangelist_remove test data.
      Parse WHITEBOARD and ERASER to hashes and then get the rangelist for
@@ -800,39 +781,39 @@ test_remove_rangelist(apr_pool_t *pool)
   struct rangelist_remove_test_data test_data[SIZE_OF_RANGE_REMOVE_TEST_ARRAY] =
     {
       /* Eraser is a proper subset of whiteboard */
-      {"/A: 1-44",  "/A: 5",  2, { {0,  4, TRUE }, {5, 44, TRUE }},
-                              2, { {0,  4, TRUE }, {5, 44, TRUE }}},
-      {"/A: 1-44*", "/A: 5",  1, { {0, 44, FALSE} },
-                              2, { {0,  4, FALSE}, {5, 44, FALSE}}},
-      {"/A: 1-44",  "/A: 5*", 1, { {0, 44, TRUE } },
-                              2, { {0,  4, TRUE }, {5, 44, TRUE }}},
-      {"/A: 1-44*", "/A: 5*", 2, { {0,  4, FALSE}, {5, 44, FALSE}},
-                              2, { {0,  4, FALSE}, {5, 44, FALSE}}},
+      {"1-44",  "5",  2, { {0,  4, TRUE }, {5, 44, TRUE }},
+                      2, { {0,  4, TRUE }, {5, 44, TRUE }}},
+      {"1-44*", "5",  1, { {0, 44, FALSE} },
+                      2, { {0,  4, FALSE}, {5, 44, FALSE}}},
+      {"1-44",  "5*", 1, { {0, 44, TRUE } },
+                      2, { {0,  4, TRUE }, {5, 44, TRUE }}},
+      {"1-44*", "5*", 2, { {0,  4, FALSE}, {5, 44, FALSE}},
+                      2, { {0,  4, FALSE}, {5, 44, FALSE}}},
       /* Non-intersecting ranges...nothing is removed */
-      {"/A: 2-9,14-19",   "/A: 12",  2, { {1, 9, TRUE }, {13, 19, TRUE }},
-                                     2, { {1, 9, TRUE }, {13, 19, TRUE }}},
-      {"/A: 2-9*,14-19*", "/A: 12",  2, { {1, 9, FALSE}, {13, 19, FALSE}},
-                                     2, { {1, 9, FALSE}, {13, 19, FALSE}}},
-      {"/A: 2-9,14-19",   "/A: 12*", 2, { {1, 9, TRUE }, {13, 19, TRUE }},
-                                     2, { {1, 9, TRUE }, {13, 19, TRUE }}},
-      {"/A: 2-9*,14-19*", "/A: 12*", 2, { {1, 9, FALSE}, {13, 19, FALSE}},
-                                     2, { {1, 9, FALSE}, {13, 19, FALSE}}},
+      {"2-9,14-19",   "12",  2, { {1, 9, TRUE }, {13, 19, TRUE }},
+                             2, { {1, 9, TRUE }, {13, 19, TRUE }}},
+      {"2-9*,14-19*", "12",  2, { {1, 9, FALSE}, {13, 19, FALSE}},
+                             2, { {1, 9, FALSE}, {13, 19, FALSE}}},
+      {"2-9,14-19",   "12*", 2, { {1, 9, TRUE }, {13, 19, TRUE }},
+                             2, { {1, 9, TRUE }, {13, 19, TRUE }}},
+      {"2-9*,14-19*", "12*", 2, { {1, 9, FALSE}, {13, 19, FALSE}},
+                             2, { {1, 9, FALSE}, {13, 19, FALSE}}},
       /* Eraser overlaps whiteboard */
-      {"/A: 1,9-17",  "/A: 12-20",  2, { {0,  1, TRUE }, {8, 11, TRUE }},
-                                    2, { {0,  1, TRUE }, {8, 11, TRUE }}},
-      {"/A: 1,9-17*", "/A: 12-20",  2, { {0,  1, TRUE }, {8, 17, FALSE}},
-                                    2, { {0,  1, TRUE }, {8, 11, FALSE}}},
-      {"/A: 1,9-17",  "/A: 12-20*", 2, { {0,  1, TRUE }, {8, 17, TRUE }},
-                                    2, { {0,  1, TRUE }, {8, 11, TRUE }}},
-      {"/A: 1,9-17*", "/A: 12-20*", 2, { {0,  1, TRUE }, {8, 11, FALSE}},
-                                    2, { {0,  1, TRUE }, {8, 11, FALSE}}},
-      /* Empty mergeinfo (i.e. empty rangelist) */
-      {"",  "",               0, { {0, 0, FALSE}},
-                              0, { {0, 0, FALSE}}},
-      {"",  "/A: 5-8,10-100", 0, { {0, 0, FALSE}},
-                              0, { {0, 0, FALSE}}},
-      {"/A: 5-8,10-100",  "", 2, { {4, 8, TRUE }, {9, 100, TRUE }},
-                              2, { {4, 8, TRUE }, {9, 100, TRUE }}}
+      {"1,9-17",  "12-20",  2, { {0,  1, TRUE }, {8, 11, TRUE }},
+                            2, { {0,  1, TRUE }, {8, 11, TRUE }}},
+      {"1,9-17*", "12-20",  2, { {0,  1, TRUE }, {8, 17, FALSE}},
+                            2, { {0,  1, TRUE }, {8, 11, FALSE}}},
+      {"1,9-17",  "12-20*", 2, { {0,  1, TRUE }, {8, 17, TRUE }},
+                            2, { {0,  1, TRUE }, {8, 11, TRUE }}},
+      {"1,9-17*", "12-20*", 2, { {0,  1, TRUE }, {8, 11, FALSE}},
+                            2, { {0,  1, TRUE }, {8, 11, FALSE}}},
+      /* Empty rangelist */
+      {"",  "",           0, { {0, 0, FALSE}},
+                          0, { {0, 0, FALSE}}},
+      {"",  "5-8,10-100", 0, { {0, 0, FALSE}},
+                          0, { {0, 0, FALSE}}},
+      {"5-8,10-100",  "", 2, { {4, 8, TRUE }, {9, 100, TRUE }},
+                          2, { {4, 8, TRUE }, {9, 100, TRUE }}}
     };
 
   err = child_err = SVN_NO_ERROR;
@@ -847,10 +828,8 @@ test_remove_rangelist(apr_pool_t *pool)
           svn_string_t *whiteboard_starting;
           svn_string_t *whiteboard_ending;
 
-          SVN_ERR(svn_mergeinfo_parse(&info1, (test_data[i]).eraser, pool));
-          SVN_ERR(svn_mergeinfo_parse(&info2, (test_data[i]).whiteboard, pool));
-          eraser = apr_hash_get(info1, "/A", APR_HASH_KEY_STRING);
-          whiteboard = apr_hash_get(info2, "/A", APR_HASH_KEY_STRING);
+          SVN_ERR(svn_rangelist__parse(&eraser, test_data[i].eraser, pool));
+          SVN_ERR(svn_rangelist__parse(&whiteboard, test_data[i].whiteboard, pool));
 
           /* Represent empty mergeinfo with an empty rangelist. */
           if (eraser == NULL)
@@ -960,7 +939,7 @@ randomly_fill_rev_array(svn_boolean_t *revs)
 /* Set *RANGELIST to a rangelist representing the revisions that are marked
  * with TRUE in the array REVS[RANDOM_REV_ARRAY_LENGTH]. */
 static svn_error_t *
-rev_array_to_rangelist(apr_array_header_t **rangelist,
+rev_array_to_rangelist(svn_rangelist_t **rangelist,
                        svn_boolean_t *revs,
                        apr_pool_t *pool)
 {
@@ -1002,7 +981,7 @@ test_rangelist_remove_randomly(apr_pool_t *pool)
       svn_boolean_t first_revs[RANDOM_REV_ARRAY_LENGTH],
         second_revs[RANDOM_REV_ARRAY_LENGTH],
         expected_revs[RANDOM_REV_ARRAY_LENGTH];
-      apr_array_header_t *first_rangelist, *second_rangelist,
+      svn_rangelist_t *first_rangelist, *second_rangelist,
         *expected_rangelist, *actual_rangelist;
       /* There will be at most RANDOM_REV_ARRAY_LENGTH ranges in
          expected_rangelist. */
@@ -1060,7 +1039,7 @@ test_rangelist_intersect_randomly(apr_pool_t *pool)
       svn_boolean_t first_revs[RANDOM_REV_ARRAY_LENGTH],
         second_revs[RANDOM_REV_ARRAY_LENGTH],
         expected_revs[RANDOM_REV_ARRAY_LENGTH];
-      apr_array_header_t *first_rangelist, *second_rangelist,
+      svn_rangelist_t *first_rangelist, *second_rangelist,
         *expected_rangelist, *actual_rangelist;
       /* There will be at most RANDOM_REV_ARRAY_LENGTH ranges in
          expected_rangelist. */
@@ -1127,7 +1106,7 @@ test_remove_mergeinfo(apr_pool_t *pool)
 static svn_error_t *
 test_rangelist_to_string(apr_pool_t *pool)
 {
-  apr_array_header_t *result;
+  svn_rangelist_t *result;
   svn_string_t *output;
   svn_string_t *expected = svn_string_create("3,5,7-11,13-14", pool);
 
@@ -1139,7 +1118,7 @@ test_rangelist_to_string(apr_pool_t *pool)
 
   SVN_ERR(svn_rangelist_to_string(&output, result, pool));
 
-  if (svn_string_compare(expected, output) != TRUE)
+  if (!svn_string_compare(expected, output))
     return fail(pool, "Rangelist string not what we expected");
 
   return SVN_NO_ERROR;
@@ -1156,7 +1135,7 @@ test_mergeinfo_to_string(apr_pool_t *pool)
 
   SVN_ERR(svn_mergeinfo_to_string(&output, info1, pool));
 
-  if (svn_string_compare(expected, output) != TRUE)
+  if (!svn_string_compare(expected, output))
     return fail(pool, "Mergeinfo string not what we expected");
 
   /* Manually construct some mergeinfo with relative path
@@ -1172,7 +1151,7 @@ test_mergeinfo_to_string(apr_pool_t *pool)
                apr_hash_get(info1, "/trunk", APR_HASH_KEY_STRING));
   SVN_ERR(svn_mergeinfo_to_string(&output, info2, pool));
 
-  if (svn_string_compare(expected, output) != TRUE)
+  if (!svn_string_compare(expected, output))
     return fail(pool, "Mergeinfo string not what we expected");
 
   return SVN_NO_ERROR;
@@ -1184,7 +1163,7 @@ test_rangelist_merge(apr_pool_t *pool)
 {
   int i;
   svn_error_t *err, *child_err;
-  apr_array_header_t *rangelist1, *rangelist2;
+  svn_rangelist_t *rangelist1, *rangelist2;
 
   /* Struct for svn_rangelist_merge test data.  Similar to
      mergeinfo_merge_test_data struct in svn_mergeinfo_merge() test. */
@@ -1201,118 +1180,118 @@ test_rangelist_merge(apr_pool_t *pool)
   struct rangelist_merge_test_data test_data[SIZE_OF_RANGE_MERGE_TEST_ARRAY] =
     {
       /* Non-intersecting ranges */
-      {"/A: 1-44",    "/A: 70-101",  2, {{ 0, 44, TRUE }, {69, 101, TRUE }}},
-      {"/A: 1-44*",   "/A: 70-101",  2, {{ 0, 44, FALSE}, {69, 101, TRUE }}},
-      {"/A: 1-44",    "/A: 70-101*", 2, {{ 0, 44, TRUE }, {69, 101, FALSE}}},
-      {"/A: 1-44*",   "/A: 70-101*", 2, {{ 0, 44, FALSE}, {69, 101, FALSE}}},
-      {"/A: 70-101",  "/A: 1-44",    2, {{ 0, 44, TRUE }, {69, 101, TRUE }}},
-      {"/A: 70-101*", "/A: 1-44",    2, {{ 0, 44, TRUE }, {69, 101, FALSE}}},
-      {"/A: 70-101",  "/A: 1-44*",   2, {{ 0, 44, FALSE}, {69, 101, TRUE }}},
-      {"/A: 70-101*", "/A: 1-44*",   2, {{ 0, 44, FALSE}, {69, 101, FALSE}}},
+      {"1-44",    "70-101",  2, {{ 0, 44, TRUE }, {69, 101, TRUE }}},
+      {"1-44*",   "70-101",  2, {{ 0, 44, FALSE}, {69, 101, TRUE }}},
+      {"1-44",    "70-101*", 2, {{ 0, 44, TRUE }, {69, 101, FALSE}}},
+      {"1-44*",   "70-101*", 2, {{ 0, 44, FALSE}, {69, 101, FALSE}}},
+      {"70-101",  "1-44",    2, {{ 0, 44, TRUE }, {69, 101, TRUE }}},
+      {"70-101*", "1-44",    2, {{ 0, 44, TRUE }, {69, 101, FALSE}}},
+      {"70-101",  "1-44*",   2, {{ 0, 44, FALSE}, {69, 101, TRUE }}},
+      {"70-101*", "1-44*",   2, {{ 0, 44, FALSE}, {69, 101, FALSE}}},
 
       /* Intersecting ranges with same starting and ending revisions */
-      {"/A: 4-20",  "/A: 4-20",  1, {{3, 20, TRUE }}},
-      {"/A: 4-20*", "/A: 4-20",  1, {{3, 20, TRUE }}},
-      {"/A: 4-20",  "/A: 4-20*", 1, {{3, 20, TRUE }}},
-      {"/A: 4-20*", "/A: 4-20*", 1, {{3, 20, FALSE}}},
+      {"4-20",  "4-20",  1, {{3, 20, TRUE }}},
+      {"4-20*", "4-20",  1, {{3, 20, TRUE }}},
+      {"4-20",  "4-20*", 1, {{3, 20, TRUE }}},
+      {"4-20*", "4-20*", 1, {{3, 20, FALSE}}},
 
       /* Intersecting ranges with same starting revision */
-      {"/A: 6-17",  "/A: 6-12",  1, {{5, 17, TRUE}}},
-      {"/A: 6-17*", "/A: 6-12",  2, {{5, 12, TRUE }, {12, 17, FALSE}}},
-      {"/A: 6-17",  "/A: 6-12*", 1, {{5, 17, TRUE }}},
-      {"/A: 6-17*", "/A: 6-12*", 1, {{5, 17, FALSE}}},
-      {"/A: 6-12",  "/A: 6-17",  1, {{5, 17, TRUE }}},
-      {"/A: 6-12*", "/A: 6-17",  1, {{5, 17, TRUE }}},
-      {"/A: 6-12",  "/A: 6-17*", 2, {{5, 12, TRUE }, {12, 17, FALSE}}},
-      {"/A: 6-12*", "/A: 6-17*", 1, {{5, 17, FALSE}}},
+      {"6-17",  "6-12",  1, {{5, 17, TRUE}}},
+      {"6-17*", "6-12",  2, {{5, 12, TRUE }, {12, 17, FALSE}}},
+      {"6-17",  "6-12*", 1, {{5, 17, TRUE }}},
+      {"6-17*", "6-12*", 1, {{5, 17, FALSE}}},
+      {"6-12",  "6-17",  1, {{5, 17, TRUE }}},
+      {"6-12*", "6-17",  1, {{5, 17, TRUE }}},
+      {"6-12",  "6-17*", 2, {{5, 12, TRUE }, {12, 17, FALSE}}},
+      {"6-12*", "6-17*", 1, {{5, 17, FALSE}}},
 
       /* Intersecting ranges with same ending revision */
-      {"/A: 5-77",   "/A: 44-77",  1, {{4, 77, TRUE }}},
-      {"/A: 5-77*",  "/A: 44-77",  2, {{4, 43, FALSE}, {43, 77, TRUE}}},
-      {"/A: 5-77",   "/A: 44-77*", 1, {{4, 77, TRUE }}},
-      {"/A: 5-77*",  "/A: 44-77*", 1, {{4, 77, FALSE}}},
-      {"/A: 44-77",  "/A: 5-77",   1, {{4, 77, TRUE }}},
-      {"/A: 44-77*", "/A: 5-77",   1, {{4, 77, TRUE }}},
-      {"/A: 44-77",  "/A: 5-77*",  2, {{4, 43, FALSE}, {43, 77, TRUE}}},
-      {"/A: 44-77*", "/A: 5-77*",  1, {{4, 77, FALSE}}},
+      {"5-77",   "44-77",  1, {{4, 77, TRUE }}},
+      {"5-77*",  "44-77",  2, {{4, 43, FALSE}, {43, 77, TRUE}}},
+      {"5-77",   "44-77*", 1, {{4, 77, TRUE }}},
+      {"5-77*",  "44-77*", 1, {{4, 77, FALSE}}},
+      {"44-77",  "5-77",   1, {{4, 77, TRUE }}},
+      {"44-77*", "5-77",   1, {{4, 77, TRUE }}},
+      {"44-77",  "5-77*",  2, {{4, 43, FALSE}, {43, 77, TRUE}}},
+      {"44-77*", "5-77*",  1, {{4, 77, FALSE}}},
 
       /* Intersecting ranges with different starting and ending revision
          where one range is a proper subset of the other. */
-      {"/A: 12-24",  "/A: 20-23",  1, {{11, 24, TRUE }}},
-      {"/A: 12-24*", "/A: 20-23",  3, {{11, 19, FALSE}, {19, 23, TRUE },
-                                       {23, 24, FALSE}}},
-      {"/A: 12-24",  "/A: 20-23*", 1, {{11, 24, TRUE }}},
-      {"/A: 12-24*", "/A: 20-23*", 1, {{11, 24, FALSE}}},
-      {"/A: 20-23",  "/A: 12-24",  1, {{11, 24, TRUE }}},
-      {"/A: 20-23*", "/A: 12-24",  1, {{11, 24, TRUE }}},
-      {"/A: 20-23",  "/A: 12-24*", 3, {{11, 19, FALSE}, {19, 23, TRUE },
-                                       {23, 24, FALSE}}},
-      {"/A: 20-23*", "/A: 12-24*", 1, {{11, 24, FALSE}}},
+      {"12-24",  "20-23",  1, {{11, 24, TRUE }}},
+      {"12-24*", "20-23",  3, {{11, 19, FALSE}, {19, 23, TRUE },
+                               {23, 24, FALSE}}},
+      {"12-24",  "20-23*", 1, {{11, 24, TRUE }}},
+      {"12-24*", "20-23*", 1, {{11, 24, FALSE}}},
+      {"20-23",  "12-24",  1, {{11, 24, TRUE }}},
+      {"20-23*", "12-24",  1, {{11, 24, TRUE }}},
+      {"20-23",  "12-24*", 3, {{11, 19, FALSE}, {19, 23, TRUE },
+                               {23, 24, FALSE}}},
+      {"20-23*", "12-24*", 1, {{11, 24, FALSE}}},
 
       /* Intersecting ranges with different starting and ending revision
          where neither range is a proper subset of the other. */
-      {"/A: 50-73",  "/A: 60-99",  1, {{49, 99, TRUE }}},
-      {"/A: 50-73*", "/A: 60-99",  2, {{49, 59, FALSE}, {59, 99, TRUE }}},
-      {"/A: 50-73",  "/A: 60-99*", 2, {{49, 73, TRUE }, {73, 99, FALSE}}},
-      {"/A: 50-73*", "/A: 60-99*", 1, {{49, 99, FALSE}}},
-      {"/A: 60-99",  "/A: 50-73",  1, {{49, 99, TRUE }}},
-      {"/A: 60-99*", "/A: 50-73",  2, {{49, 73, TRUE }, {73, 99, FALSE}}},
-      {"/A: 60-99",  "/A: 50-73*", 2, {{49, 59, FALSE}, {59, 99, TRUE }}},
-      {"/A: 60-99*", "/A: 50-73*", 1, {{49, 99, FALSE}}},
+      {"50-73",  "60-99",  1, {{49, 99, TRUE }}},
+      {"50-73*", "60-99",  2, {{49, 59, FALSE}, {59, 99, TRUE }}},
+      {"50-73",  "60-99*", 2, {{49, 73, TRUE }, {73, 99, FALSE}}},
+      {"50-73*", "60-99*", 1, {{49, 99, FALSE}}},
+      {"60-99",  "50-73",  1, {{49, 99, TRUE }}},
+      {"60-99*", "50-73",  2, {{49, 73, TRUE }, {73, 99, FALSE}}},
+      {"60-99",  "50-73*", 2, {{49, 59, FALSE}, {59, 99, TRUE }}},
+      {"60-99*", "50-73*", 1, {{49, 99, FALSE}}},
 
       /* Multiple ranges. */
-      {"/A: 1-5,7,12-13",    "/A: 2-17",  1, {{0,  17, TRUE }}},
-      {"/A: 1-5*,7*,12-13*", "/A: 2-17*", 1, {{0,  17, FALSE}}},
+      {"1-5,7,12-13",    "2-17",  1, {{0,  17, TRUE }}},
+      {"1-5*,7*,12-13*", "2-17*", 1, {{0,  17, FALSE}}},
 
-      {"/A: 1-5,7,12-13",    "/A: 2-17*", 6,
+      {"1-5,7,12-13",    "2-17*", 6,
        {{0,  5, TRUE }, { 5,  6, FALSE}, { 6,  7, TRUE },
         {7, 11, FALSE}, {11, 13, TRUE }, {13, 17, FALSE}}},
 
-      {"/A: 1-5*,7*,12-13*", "/A: 2-17", 2,
+      {"1-5*,7*,12-13*", "2-17", 2,
        {{0, 1, FALSE}, {1, 17, TRUE }}},
 
-      {"/A: 2-17",  "/A: 1-5,7,12-13",    1, {{0,  17, TRUE }}},
-      {"/A: 2-17*", "/A: 1-5*,7*,12-13*", 1, {{0,  17, FALSE}}},
+      {"2-17",  "1-5,7,12-13",    1, {{0,  17, TRUE }}},
+      {"2-17*", "1-5*,7*,12-13*", 1, {{0,  17, FALSE}}},
 
-      {"/A: 2-17*", "/A: 1-5,7,12-13", 6,
+      {"2-17*", "1-5,7,12-13", 6,
        {{0,  5, TRUE }, { 5,  6, FALSE}, { 6,  7, TRUE },
         {7, 11, FALSE}, {11, 13, TRUE }, {13, 17, FALSE}}},
 
-      {"/A: 2-17", "/A: 1-5*,7*,12-13*", 2,
+      {"2-17", "1-5*,7*,12-13*", 2,
        {{0, 1, FALSE}, {1, 17, TRUE}}},
 
-      {"/A: 3-4*,10-15,20", "/A: 5-60*", 5,
+      {"3-4*,10-15,20", "5-60*", 5,
        {{2, 9, FALSE}, {9, 15, TRUE}, {15, 19, FALSE},{19, 20, TRUE},
         {20, 60, FALSE}}},
 
-      {"/A: 5-60*", "/A: 3-4*,10-15,20", 5,
+      {"5-60*", "3-4*,10-15,20", 5,
        {{2, 9, FALSE}, {9, 15, TRUE}, {15, 19, FALSE},{19, 20, TRUE},
         {20, 60, FALSE}}},
 
-      {"/A: 3-4*,50-100*", "/A: 5-60*", 1, {{2, 100, FALSE}}},
+      {"3-4*,50-100*", "5-60*", 1, {{2, 100, FALSE}}},
 
-      {"/A: 5-60*", "/A: 3-4*,50-100*", 1, {{2, 100, FALSE}}},
+      {"5-60*", "3-4*,50-100*", 1, {{2, 100, FALSE}}},
 
-      {"/A: 3-4*,50-100", "/A: 5-60*", 2, {{2, 49, FALSE}, {49, 100, TRUE}}},
+      {"3-4*,50-100", "5-60*", 2, {{2, 49, FALSE}, {49, 100, TRUE}}},
 
-      {"/A: 5-60*", "/A: 3-4*,50-100", 2, {{2, 49, FALSE}, {49, 100, TRUE}}},
+      {"5-60*", "3-4*,50-100", 2, {{2, 49, FALSE}, {49, 100, TRUE}}},
 
-      {"/A: 3-4,50-100*", "/A: 5-60", 2, {{2, 60, TRUE}, {60, 100, FALSE}}},
+      {"3-4,50-100*", "5-60", 2, {{2, 60, TRUE}, {60, 100, FALSE}}},
 
-      {"/A: 5-60", "/A: 3-4,50-100*", 2, {{2, 60, TRUE}, {60, 100, FALSE}}},
+      {"5-60", "3-4,50-100*", 2, {{2, 60, TRUE}, {60, 100, FALSE}}},
 
-      {"/A: 5,9,11-15,17,200-300,999", "/A: 7-50", 4,
+      {"5,9,11-15,17,200-300,999", "7-50", 4,
        {{4, 5, TRUE}, {6, 50, TRUE}, {199, 300, TRUE}, {998, 999, TRUE}}},
 
       /* A rangelist merged with an empty rangelist should equal the
          non-empty rangelist but in compacted form. */
-      {"/A: 1-44,45,46,47-50",       "",  1, {{ 0, 50, TRUE }}},
-      {"/A: 1,2,3,4,5,6,7,8",        "",  1, {{ 0, 8,  TRUE }}},
-      {"/A: 6-10,12-13,14,15,16-22", "",  2,
+      {"1-44,45,46,47-50",       "",  1, {{ 0, 50, TRUE }}},
+      {"1,2,3,4,5,6,7,8",        "",  1, {{ 0, 8,  TRUE }}},
+      {"6-10,12-13,14,15,16-22", "",  2,
        {{ 5, 10, TRUE }, { 11, 22, TRUE }}},
-      {"", "/A: 1-44,45,46,47-50",        1, {{ 0, 50, TRUE }}},
-      {"", "/A: 1,2,3,4,5,6,7,8",         1, {{ 0, 8,  TRUE }}},
-      {"", "/A: 6-10,12-13,14,15,16-22",  2,
+      {"", "1-44,45,46,47-50",        1, {{ 0, 50, TRUE }}},
+      {"", "1,2,3,4,5,6,7,8",         1, {{ 0, 8,  TRUE }}},
+      {"", "6-10,12-13,14,15,16-22",  2,
        {{ 5, 10, TRUE }, { 11, 22, TRUE }}},
 
       /* An empty rangelist merged with an empty rangelist is, drum-roll
@@ -1325,10 +1304,8 @@ test_rangelist_merge(apr_pool_t *pool)
     {
       svn_string_t *rangelist2_starting, *rangelist2_ending;
 
-      SVN_ERR(svn_mergeinfo_parse(&info1, (test_data[i]).mergeinfo1, pool));
-      SVN_ERR(svn_mergeinfo_parse(&info2, (test_data[i]).mergeinfo2, pool));
-      rangelist1 = apr_hash_get(info1, "/A", APR_HASH_KEY_STRING);
-      rangelist2 = apr_hash_get(info2, "/A", APR_HASH_KEY_STRING);
+      SVN_ERR(svn_rangelist__parse(&rangelist1, test_data[i].mergeinfo1, pool));
+      SVN_ERR(svn_rangelist__parse(&rangelist2, test_data[i].mergeinfo2, pool));
 
       /* Create empty rangelists if necessary. */
       if (rangelist1 == NULL)
@@ -1383,7 +1360,7 @@ test_rangelist_diff(apr_pool_t *pool)
 {
   int i;
   svn_error_t *err, *child_err;
-  apr_array_header_t *from, *to, *added, *deleted;
+  svn_rangelist_t *from, *to, *added, *deleted;
 
   /* Structure containing two ranges to diff and the expected output of the
      diff both when considering and ignoring range inheritance. */
@@ -1412,7 +1389,7 @@ test_rangelist_diff(apr_pool_t *pool)
   #define SIZE_OF_RANGE_DIFF_TEST_ARRAY 16
   /* The actual test data array.
 
-                    'from' --> {"/A: 1,5-8",  "/A: 1,6,10-12", <-- 'to'
+                    'from' --> {"1,5-8",  "1,6,10-12", <-- 'to'
       Number of adds when  -->  1, { { 9, 12, TRUE } },
       considering inheritance
 
@@ -1430,97 +1407,97 @@ test_rangelist_diff(apr_pool_t *pool)
   struct rangelist_diff_test_data test_data[SIZE_OF_RANGE_DIFF_TEST_ARRAY] =
     {
       /* Add and Delete */
-      {"/A: 1",  "/A: 3",
+      {"1",  "3",
        1, { { 2, 3, TRUE } },
        1, { { 0, 1, TRUE } },
        1, { { 2, 3, TRUE } },
        1, { { 0, 1, TRUE } } },
 
       /* Add only */
-      {"/A: 1",  "/A: 1,3",
+      {"1",  "1,3",
        1, { { 2, 3, TRUE } },
        0, { { 0, 0, FALSE } },
        1, { { 2, 3, TRUE } },
        0, { { 0, 0, FALSE } } },
 
       /* Delete only */
-      {"/A: 1,3",  "/A: 1",
+      {"1,3",  "1",
        0, { { 0, 0, FALSE } },
        1, { { 2, 3, TRUE  } },
        0, { { 0, 0, FALSE } },
        1, { { 2, 3, TRUE  } } },
 
       /* No diff */
-      {"/A: 1,3",  "/A: 1,3",
+      {"1,3",  "1,3",
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } } },
 
-      {"/A: 1,3*",  "/A: 1,3*",
+      {"1,3*",  "1,3*",
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } } },
 
       /* Adds and Deletes */
-      {"/A: 1,5-8",  "/A: 1,6,10-12",
+      {"1,5-8",  "1,6,10-12",
        1, { { 9, 12, TRUE } },
        2, { { 4, 5, TRUE }, { 6, 8, TRUE } },
        1, { { 9, 12, TRUE } },
        2, { { 4, 5, TRUE }, { 6, 8, TRUE } } },
 
-      {"/A: 6*",  "/A: 6",
+      {"6*",  "6",
        1, { { 5, 6, TRUE  } },
        1, { { 5, 6, FALSE } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } } },
 
       /* Intersecting range with different inheritability */
-      {"/A: 6",  "/A: 6*",
+      {"6",  "6*",
        1, { { 5, 6, FALSE } },
        1, { { 5, 6, TRUE  } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } } },
 
-      {"/A: 6*",  "/A: 6",
+      {"6*",  "6",
        1, { { 5, 6, TRUE  } },
        1, { { 5, 6, FALSE } },
        0, { { 0, 0, FALSE } },
        0, { { 0, 0, FALSE } } },
 
-      {"/A: 1,5-8",  "/A: 1,6*,10-12",
+      {"1,5-8",  "1,6*,10-12",
        2, { { 5,  6, FALSE }, { 9, 12, TRUE } },
        1, { { 4,  8, TRUE  } },
        1, { { 9, 12, TRUE  } },
        2, { { 4,  5, TRUE  }, { 6,  8, TRUE } } },
 
-     {"/A: 1,5-8*",  "/A: 1,6,10-12",
+     {"1,5-8*",  "1,6,10-12",
        2, { { 5,  6, TRUE  }, { 9, 12, TRUE  } },
        1, { { 4,  8, FALSE } },
        1, { { 9, 12, TRUE  } },
        2, { { 4,  5, FALSE }, { 6,  8, FALSE } } },
 
       /* Empty range diffs */
-      {"/A: 3-9",  "",
+      {"3-9",  "",
        0, { { 0, 0, FALSE } },
        1, { { 2, 9, TRUE  } },
        0, { { 0, 0, FALSE } },
        1, { { 2, 9, TRUE  } } },
 
-      {"/A: 3-9*",  "",
+      {"3-9*",  "",
        0, { { 0, 0, FALSE } },
        1, { { 2, 9, FALSE } },
        0, { { 0, 0, FALSE } },
        1, { { 2, 9, FALSE } } },
 
-      {"",  "/A: 3-9",
+      {"",  "3-9",
        1, { { 2, 9, TRUE  } },
        0, { { 0, 0, FALSE } },
        1, { { 2, 9, TRUE  } },
        0, { { 0, 0, FALSE } } },
 
-      {"",  "/A: 3-9*",
+      {"",  "3-9*",
        1, { { 2, 9, FALSE } },
        0, { { 0, 0, FALSE } },
        1, { { 2, 9, FALSE } },
@@ -1537,10 +1514,8 @@ test_rangelist_diff(apr_pool_t *pool)
   err = child_err = SVN_NO_ERROR;
   for (i = 0; i < SIZE_OF_RANGE_DIFF_TEST_ARRAY; i++)
     {
-      SVN_ERR(svn_mergeinfo_parse(&info1, (test_data[i]).to, pool));
-      SVN_ERR(svn_mergeinfo_parse(&info2, (test_data[i]).from, pool));
-      to = apr_hash_get(info1, "/A", APR_HASH_KEY_STRING);
-      from = apr_hash_get(info2, "/A", APR_HASH_KEY_STRING);
+      SVN_ERR(svn_rangelist__parse(&to, test_data[i].to, pool));
+      SVN_ERR(svn_rangelist__parse(&from, test_data[i].from, pool));
 
       /* Represent empty mergeinfo with an empty rangelist. */
       if (to == NULL)
@@ -1721,7 +1696,7 @@ struct svn_test_descriptor_t test_funcs[] =
     SVN_TEST_PASS2(test_diff_mergeinfo,
                    "diff of mergeinfo"),
     SVN_TEST_PASS2(test_merge_mergeinfo,
-                   "merging of mergeinfo hashs"),
+                   "merging of mergeinfo hashes"),
     SVN_TEST_PASS2(test_mergeinfo_intersect,
                    "intersection of mergeinfo"),
     SVN_TEST_PASS2(test_rangelist_to_string,
