@@ -28,6 +28,6 @@
 CONFIG_DIR=${1-$HOME/.subversion}
 for i in $CONFIG_DIR/auth/svn.ssl.server/????????????????????????????????; do
   grep :// $i
-  grep '.\{80\}' $i | sed 's/\(.\{64\}\)/\1\n/g' | openssl base64 -d | openssl x509 -inform der -noout -fingerprint | sed 's/=/\n/'
+  grep '.\{80\}' $i | sed 's/\(.\{64\}\)/\1 /g' | xargs -n1 | openssl base64 -d | openssl x509 -inform der -noout -fingerprint | sed 's/=/ /' | xargs -n1
   echo
 done
